#!/usr/bin/ruby
# encoding: utf-8

class AntibotAviraScanner
	
	def initialize(drvscreen, params, logdir=nil)
		@drivescreen = drvscreen 
		@params = params
		@logdir = logdir
		@qparams = params.collect { |x| "'#{x}'" }  
		@infections = Hash.new
		@killed = false
		@partcount = -1
	end
	attr_reader :infections, :killed
	
	def kill_scan
		system("killall -9 scancl")
		@killed = true
	end
	
	def prepare_scan(pgbar=nil, label=nil)
		puts "Preparing scan:"
		now = Time.new.strftime("%Y%m%d-%H%M%S") 
		system("blkid | grep -v squashfs > '#{@logdir}/#{now}-laufwerke.txt' ") unless @logdir.nil?
		@partcount = 0
		@drivescreen.dump_marked
		@drivescreen.int_drives.each { |d| 
			d.partitions.each { |p|
				if @drivescreen.saved_states[p.uuid] == true
					puts "Marked for scan: " + p.uuid
					p.filecount(pgbar)
					@partcount += 1
				end
			}
		}
		@drivescreen.ext_drives.each { |d|
			d.partitions.each { |p|
				if @drivescreen.saved_states[p.uuid] == true
					puts "Marked for scan: " + p.uuid
					p.filecount(pgbar)
					@partcount += 1
				end
			}
		}
		pgbar.text = @partcount.to_s + " Partitionen zu scannen"
	end
	
	# interval: delta in seconds (integer) for the signature update
	# FIXME! Use proxy settings
	# FIXME! Change avupdate script to return proper exit code!
	
	def update_signatures(interval, pgbar=nil)
		return false, "Schlüsseldatei fehlt!" unless File.exist?("/AntiVir/rescue_cd.key")
		return true, nil if File.stat("/AntiVir/rescue_cd.key").mtime.to_f + interval.to_f > Time.now.to_f 
		lastupd = Time.now.to_f
		now = Time.new.strftime("%Y%m%d-%H%M%S") 
		log = File.new(@logdir + "/" + now + "-avupdate.txt", "w") unless @logdir.nil?
		IO.popen("ruby avupdate_wrapper.rb /var/run/avupdate.ret") { |l|
			while l.gets
				line = $_
				unless pgbar.nil? || Time.now.to_f - lastupd < 0.1
					lastupd = Time.now.to_f
					pgbar.pulse
					pgbar.text = "Aktualisiere Virensignaturen"
					while (Gtk.events_pending?)
						Gtk.main_iteration
					end
				end
				unless line.strip == "-TickTock-" 
					puts line
					log.write(line) unless @logdir.nil?
				end
			end
		}
		log.close unless @logdir.nil?
		retval = -1
		begin
			retval = File.read("/var/run/avupdate.ret").strip.to_i
		rescue
		end
		puts "avupdate returned: " + retval.to_s
		unless pgbar.nil?
			pgbar.text = "Aktualisierung der Virensignaturen abgeschlossen"
			pgbar.fraction = 1.0
		end
		if retval == 0
			# touch the key file
			system("touch /AntiVir/rescue_cd.key")
			return true, nil
		elsif retval == 1
			# no update available - do nothing
			return true, nil
		else
			# Update failed
			# FIXME! Show dialog informing on the problem
			return false, "Die Aktualisierung der Virensignaturen ist fehlgeschlagen. Wahrscheinlichste Ursache ist eine fehlerhafte Netzwerkkonfiguration."
		end
	end
	
	# FIXME: Use wrapper script
	# FIXME: Use settings!
	# FIXME: Proper handling of boot sector scan
	
	def scan_partition(p, pc=0, pgbar=nil, label=nil)
		return false if @killed == true
		p.filecount(pgbar)
		p.mount 
		filect = 0
		lastfname = nil
		infected = Array.new
		last_pulse = Time.now.to_f
		unless pgbar.nil?
			pgbar.text = "Partition " + pc.to_s + " von " + @partcount.to_s + " - Starte Scan"
			pgbar.fraction = 0.0 
			while (Gtk.events_pending?)
				Gtk.main_iteration
			end
		end
		# run ruby scancl_wrapper.rb instead 
		# IO.popen("/AntiVir/scancl -a -z -s --showall --log=/dev/null '" + p.mount_point[0] + "'") { |l|
		scancl = "ruby scancl_wrapper.rb /var/run/scancl.#{p.uuid}.ret /AntiVir/scancl " + @qparams.join(" ") + " "
		scancl = scancl + "'#{p.mount_point[0]}'" unless @params[-1] =~/^\// 
		puts "Running command: " + scancl
		now = Time.new.strftime("%Y%m%d-%H%M%S") 
		log = File.new(@logdir + "/" + now + "-" + p.device + "-botscan.txt", "w") unless @logdir.nil?
		puts "Logging to: " + @logdir + "/" + now + "-" + p.device + "-botscan.txt"
		prefix = Array.new
		prefix_finished = false
		suffix = Array.new
		suffix_started = false
		file_lines = Array.new
		alert = false
		IO.popen(scancl) { |l|
			while l.gets
				now = Time.now.to_f
				unless pgbar.nil? || now - last_pulse < 0.1 
					last_pulse = now
					pgbar.text = "Partition " + pc.to_s + " von " + @partcount.to_s + " - " + filect.to_s + " Dateien gescannt"
					# puts "Partition " + pc.to_s + " von " + @partcount.to_s + " - " + filect.to_s + " Dateien gescannt"
					unless p.filecount(pgbar).nil?
						if filect.to_i > 0
							pgbar.fraction = filect.to_f / p.filecount(pgbar).to_f
						else 
							pgbar.pulse
						end
					end
					while (Gtk.events_pending?)
						Gtk.main_iteration
					end
				end
				line = $_
				
				if prefix_finished == false
					if line.strip =~ /^\//
						prefix_finished = true
						prefix.each { |l| log.write(l) }
						0.upto(3) { |n| log.write("\n") }
						# puts "FINISHED PREFIX - FLUSHING:"
						# prefix.each { |l| $stdout.write(l) }
					else
						prefix.push(line) unless line.strip =~ /^-TickTock-$/
					end
				end
				
				if line.strip =~ /^Statistics/ || suffix_started == true
					if line.strip =~ /^Statistics/ && alert == true
						# Flush remaining alert
						file_lines.each { |l| log.write(l) }
						alert = false
					end
					if line.strip =~ /^Statistics/
						0.upto(3) { |n| log.write("\n") }
					end
					unless line.strip =~ /^-TickTock-$/
						log.write(line) 
						suffix.push(line)
					end
					suffix_started = true
				end
				
				if line.strip =~ /^\//
					lastfname = line.strip 
					filect += 1
					if file_lines.size > 0 && alert == true
						file_lines.each { |l| log.write(l) }
						alert = false
					end
					file_lines = Array.new
					file_lines.push(line)
				elsif (line.strip =~ /^-TickTock-$/ ).nil? 
					if line.strip =~ /^ALERT/
						alert = true 
						infected.push AntibotInfectedFile.new(lastfname, p.mount_point[0], p.uuid, line)
						puts line.strip 
					end
					file_lines.push(line) unless suffix_started == true || prefix_finished == false 
				end
				
			end
		}
		log.close unless @logdir.nil?
		unless pgbar.nil?
			pgbar.text = "Partition " + pc.to_s + " von " + @partcount.to_s + " - Scan abgeschlossen"
			pgbar.fraction = 1.0 
			while (Gtk.events_pending?)
				Gtk.main_iteration
			end
		end
		p.umount
		@infections[p] = infected
		# infected.each { |i|
		#	puts "File: " + i[0] + " Infection: " + i[1] 
		# }
		return false if @killed == true
	end
	
	def run_scan(pgbar=nil, label=nil)
		# Reset variable for kill
		@killed = false
		pc = 1
		@drivescreen.int_drives.each { |d|
			d.partitions.each { |p|
				if @drivescreen.saved_states[p.uuid] == true
					scan_partition(p, pc, pgbar, label)
					pc += 1
				end
			}
		}
		@drivescreen.ext_drives.each { |d|
			d.partitions.each { |p|
				if @drivescreen.saved_states[p.uuid] == true
					scan_partition(p, pc, pgbar, label)
					pc += 1
				end
			}
		}
	end
	
	def dump_scan_results
		
	end
	
end
