#!/usr/bin/ruby
# encoding: utf-8

require "MfsDiskDrive.rb"
require "MfsSinglePartition.rb"

class AntibotDriveScreen
	
	def initialize
		@wdgt = Gtk::VBox.new(false, 6)
		@drives = Array.new
		@int_drives = Array.new
		@ext_drives = Array.new
		@int_buttons = Hash.new
		@ext_buttons = Hash.new
		@saved_states = Hash.new
		@intlabel = nil
		@extlabel = nil
		@int_vbox = Gtk::VBox.new(false, 3)
		@ext_vbox = Gtk::VBox.new(false, 3)
		@wdgt.pack_start(Gtk::Label.new("Welche Laufwerke wollen Sie untersuchen?") , false, false, 10)
		reread_drivelist 
	end
	
	attr_reader :wdgt, :ext_drives, :int_drives, :saved_states
	attr_accessor :int_buttons, :ext_buttons # , :int_vbox, :ext_vbox

	def reread_drivelist
		# @wdgt.destroy 
		# @wdgt = Gtk::VBox.new(false, 6)
		@drives = Array.new
		Dir.entries("/sys/block").each { |l|
			@drives.push(MfsDiskDrive.new(l, true)) if l =~ /[a-z]$/ 
		}
		@int_drives = Array.new
		@ext_drives = Array.new
		@int_buttons.each { |k,v|
			v.destroy
			@int_buttons.delete(k)
		}
		@ext_buttons.each { |k,v|
			v.destroy
			@ext_buttons.delete(k)
		}
		@intlabel.destroy unless @intlabel.nil?
		@extlabel.destroy unless @extlabel.nil?
		@intlabel = Gtk::Label.new("<b>Interne Laufwerke</b>")
		@intlabel.use_markup = true
		@extlabel = Gtk::Label.new("<b>Externe Laufwerke</b>")
		@extlabel.use_markup = true
		@drives.each { |d|
			if d.usb == true
				@ext_drives.push(d)
			else
				@int_drives.push(d)
			end
		}
		intparts = 0
		@int_drives.each { |d|
			d.partitions.each { |p|
				intparts += create_button(d, p, true)
			}
		}
		extparts = 0
		@ext_drives.each { |d|
			d.partitions.each { |p|
				extparts += create_button(d, p, false)
			}
		}
		if intparts > 0 
			@wdgt.pack_start(@intlabel, false, false, 0)
			@int_buttons.each { |k,v|
				puts "Adding button for UUID: " + k
				@wdgt.pack_start(v, false, false, 0)
			}
		end
		if extparts > 0 
			@wdgt.pack_start(@extlabel, false, false, 0)
			@ext_buttons.each { |k,v|
				puts "Adding button for UUID: " + k
				@wdgt.pack_start(v, false, false, 0)
			}
		end
		@wdgt.show_all
		dump_marked
	end

	def dump_marked
		@saved_states.each { |k,v|
			puts "UUID: " + k + " marked for scan " + v.to_s 
		}
	end
	
	def create_button(d, p, internal=true)
		if p.fs =~ /ext/ || p.fs =~ /fat/ || p.fs =~ /ntfs/
			button_text = d.device + ", Partition " + p.device.to_s + ", " + p.fs.to_s + " " + p.label.to_s + " " + p.human_size
			button_text = d.device + " (" + d.vendor + " " + d.model + "), Partition " + p.device.to_s + ", " + p.fs.to_s + 
				" " + p.label.to_s + " " + p.human_size if internal == false
			button = Gtk::CheckButton.new
			button.active = true
			if !p.mount_point.nil? && p.mount_point[0] =~ /^\/lesslinux\// 
				button_text = button_text + " (Systemlaufwerk)"
				button.active = false 
				@saved_states[p.uuid] = false
				button.sensitive = false # if p.mount_point[0] =~ /^\/lesslinux\/antibot/ 
			end
			button.label = button_text
			if @saved_states.has_key?(p.uuid) 
				button.active = @saved_states[p.uuid] 
			else
				@saved_states[p.uuid] = button.active?
			end
			button.signal_connect("clicked") {
				if button.active? == false
					puts "Remove " + p.uuid
					@saved_states[p.uuid] = false
				else
					puts "Add " + p.uuid
					@saved_states[p.uuid] = true
				end
				dump_marked
			}
			puts "Adding button (int) for " + p.uuid 
			@int_buttons[p.uuid] = button if internal == true
			@int_buttons[p.uuid] = button if internal == false
			return 1
		end
		return 0
	end

end