#!/usr/bin/ruby
# encoding: utf-8

require "resolv"

class AntibotPrereqScreen
	
	def initialize(assi)
		@wdgt = Gtk::Alignment.new(0.5, 0.5, 0.8, 0.0)
		@assistant = assi
		@complete = false
		# Proxy
		@proxy_settings = [ "", "", "", "" ]
		# Icons
		@icon = Array.new
		0.upto(2) { |i| @icon.push Gtk::Image.new(Gtk::Stock::CANCEL, Gtk::IconSize::BUTTON)}
		# Short labels
		@slab = [ Gtk::Label.new("<b>Netzwerk</b>"),
			Gtk::Label.new("<b>Energie</b>"),
			Gtk::Label.new("<b>Speicher</b>") ]
		@slab.each { |s|
			s.use_markup = true
		}
		# Long descriptions
		@desc = Array.new
		0.upto(2) { |i| 
			@desc[i] = Gtk::Label.new("Hier steht ein etwas längerer Text, der möglicherweise über mehrere Zeilen geht. Dies ist Blindtext.")
			@desc[i].width_request = 250
			@desc[i].wrap = true 
		}
		# Buttons
		@buttons = Array.new
		@buttons[0] = Gtk::Button.new("Netzwerk einrichten")
		@buttons[1] = Gtk::Button.new("Erneut prüfen")
		@buttons[2] = Gtk::Button.new("USB-Stick einrichten")
		@buttboxes = Array.new
		0.upto(2) { |i|  
			@buttboxes[i] = Gtk::Alignment.new(0.5, 0.5, 1.0, 0.0) 
			@buttboxes[i].add @buttons[i]
		}
		
		# Table
		stab = Gtk::Table.new(4, 3, false)
		stab.set_row_spacings 7
		stab.set_column_spacings 5
		0.upto(@slab.size - 1) { |i|
			stab.attach_defaults(@icon[i], 0, 1, i, i + 1)
			stab.attach_defaults(@slab[i], 1, 2, i, i + 1)
			stab.attach_defaults(@buttboxes[i], 3, 4, i, i + 1)
			stab.attach_defaults(@desc[i], 2, 3, i, i + 1)
		}
		@buttons[1].signal_connect('clicked') { check_power }
		check_power
		@buttons[0].signal_connect('clicked') { 
			net_dialog
			check_net
		}
		check_net
		@buttons[2].signal_connect('clicked') { 
			usb_dialog
			check_memory
		}
		check_memory
		@wdgt.add stab
		
		
	end
	attr_reader :wdgt, :complete
	
	def check_power
		unless File.exist?("/proc/acpi/battery/BAT0/state") 
			@icon[1].set(Gtk::Stock::APPLY, Gtk::IconSize::BUTTON)
			@desc[1].text = "Ihr Computer ist mit der Energieversorgung verbunden."
			return 
		end
		File.open("/proc/acpi/battery/BAT0/state").each { |line|
			ltoks = line.strip.split(':')
			if ltoks[0].strip =~ /charging/ && ltoks[1].strip =~ /discharging/ 
				@icon[1].set(Gtk::Stock::DIALOG_WARNING, Gtk::IconSize::BUTTON)
				@desc[1].text = "Bitte schließen Sie Ihr Notebook an das Stromnetz an."
				return 
			elsif ltoks[0].strip =~ /charging/ && ltoks[1].strip =~ /^charging/ 
				@icon[1].set(Gtk::Stock::APPLY, Gtk::IconSize::BUTTON)
				@desc[1].text = "Ihr Computer ist mit der Stromversorgung verbunden."
				return 
			end
		}
	end
	
	def net_dialog
		frame_width = 640
		# Create the dialog
		dialog = Gtk::Dialog.new("Netzwerk",
                             $main_application_window,
                             Gtk::Dialog::MODAL,
                             [ Gtk::Stock::OK, Gtk::Dialog::RESPONSE_NONE ])
		ovbox = Gtk::VBox.new(false, 6)
		frames = Array.new
		[ "WLAN und IP-Adressen", "Proxy-Server" ].each { |s| frames.push(Gtk::Frame.new(s)) }
		netlabel = Gtk::Label.new("Sie können den <i>WiCD Network Manager</i> verwenden, um mit einem WLAN zu verbinden oder statische IP-Adressen zu vergeben")
		netlabel.use_markup = true
		netlabel.wrap = true
		netlabel.width_request = 450
		wicd_button = Gtk::Button.new("WiCD Network Manager starten")
		wicd_button.signal_connect('clicked') { system("wicd-gtk --no-tray &") } 
		netbox = Gtk::VBox.new(false, 6)
		netbox.pack_start(netlabel, true, false, 3)
		netbox.pack_start(wicd_button, true, false, 3)
		frames[0].add netbox
		proxylabel = Gtk::Label.new("Tragen Sie hier die Proxy-Zugangsdaten ein, falls ein Proxy zur Aktualisierung der Virensignaturen benötigt wird")
		proxylabel.use_markup = true
		proxylabel.wrap = true
		proxylabel.width_request = 450
		proxytab = Gtk::Table.new( 2, 4, false)
		proxydesc = [ "IP oder Hostname:", "Port:", "Nutzername:", "Passwort:" ]
		proxyfields = Array.new
		0.upto(3) { |n|
			proxytab.attach(Gtk::Label.new(proxydesc[n]), 0, 1, n, n+1)
			proxyfields[n] = Gtk::Entry.new
			proxyfields[n].text = @proxy_settings[n].strip
			proxytab.attach(proxyfields[n], 1, 2, n, n+1) 
		}
		proxybox = Gtk::VBox.new(false, 6)
		proxybox.pack_start(proxylabel, true, false, 3)
		proxybox.pack_start(proxytab, true, false, 3)
		frames[1].add proxybox
		frames.each { |f|
			ovbox.pack_start(f, true, true, 3)
		}
		dialog.vbox.add(ovbox)
		dialog.signal_connect('response') { 
			0.upto(3) { |n| @proxy_settings[n] = proxyfields[n].text.strip }
			check_net
			dialog.destroy
		}
		dialog.show_all
	end
	
	# FIXME: For now just try name resolution! - Must be extended to check if download via proxy works
	def check_net
		if @proxy_settings[0].strip.size > 0
			@icon[0].set(Gtk::Stock::APPLY, Gtk::IconSize::BUTTON)
			@desc[0].text = "Die gefundene Internetverbindung kann zur Aktualisierung der Signaturen benutzt werden."
			return true
		end
		dns = Resolv::DNS.new
		addr = nil
		begin
			addr = dns.getaddress("www.avira.com")
		rescue
		end
		if addr.nil?
			@icon[0].set(Gtk::Stock::DIALOG_WARNING, Gtk::IconSize::BUTTON)
			@desc[0].text = "Bitte richten Sie Ihr Netzwerk ein damit die Aktualiserung der Signaturen möglich ist."
		else
			@icon[0].set(Gtk::Stock::APPLY, Gtk::IconSize::BUTTON)
			@desc[0].text = "Die gefundene Internetverbindung kann zur Aktualisierung der Signaturen benutzt werden."
		end
	end
	
	def check_memory
		puts "INFO: (re-)checking memory"
		mem = -1
		File.open("/proc/meminfo").each { |line|
			ltoks = line.strip.split
			mem = ltoks[1].to_i if ltoks[0] =~ /MemTotal/  
		}
		if system("mountpoint /lesslinux/antibot") || system("mountpoint /tmp/lesslinux/antibot")
			@icon[2].set(Gtk::Stock::APPLY, Gtk::IconSize::BUTTON)
			@desc[2].text = "Sie verwenden einen USB-Speicherstift für Virensignaturen, Protokolle und die Auslagerungsdatei."
			@buttons[2].sensitive = false
			set_complete(true)
		elsif mem > 1572863
			@icon[2].set(Gtk::Stock::DIALOG_WARNING, Gtk::IconSize::BUTTON)
			@desc[2].text = "Es ist kein USB-Speicherstift für Antibot 3.0 angeschlossen. Wir empfehlen, einen einzurichten."
			set_complete(true)
		else
			@icon[2].set(Gtk::Stock::CANCEL, Gtk::IconSize::BUTTON)
			@desc[2].text = "Aufgrund des geringen Hauptspeichers ist die Verwendung eines USB-Sticks zwingend erforderlich."
			set_complete(false)
		end
	end
	
	def set_complete(comp)
		@assistant.set_page_complete(@wdgt, comp)
		@complete = comp
	end
	
	def usb_dialog
		frame_width = 640
		# Create the dialog
		dialog = Gtk::Dialog.new("USB-Installation",
                             $main_application_window,
                             Gtk::Dialog::MODAL,
                             [ Gtk::Stock::OK, Gtk::Dialog::RESPONSE_NONE ])
		ovbox = Gtk::VBox.new(false, 6)
		frames = Array.new
		[ "Ziel", "Methode", "Installation" ].each { |s| frames.push(Gtk::Frame.new(s)) }
		tgtvbox = Gtk::VBox.new(false, 6)
		tgtlabel = Gtk::Label.new("Wählen Sie den USB-Speicherstift oder die Speicherkarte, auf der Sie Antibot 3.0 installieren wollen")
		tgtlabel.width_request = 450
		tgtlabel.wrap = true
		tgthbox = Gtk::HBox.new(false, 3)
		tgtsel = Gtk::ComboBox.new
		targets = fill_drivelist(tgtsel)
		tgthbox.pack_start(tgtsel, true, true, 3)
		tgtreread = Gtk::Button.new("Neu einlesen") 
		tgtreread.signal_connect("clicked") {
			targets = fill_drivelist(tgtsel)
		}
		tgthbox.pack_start(tgtreread, false, false, 3)
		tgtvbox.pack_start(tgtlabel, true, false, 3)
		tgtvbox.pack_start(tgthbox, true, false, 3)
		frames[0].add tgtvbox
		methvbox = Gtk::VBox.new(false, 6)
		methpartradio = Gtk::RadioButton.new( "Nur Signaturen und Protokolle: Vorhandene Daten bleiben bestehen")
		methfullradio = Gtk::RadioButton.new(methpartradio, "Startfähig: Vorhandene Daten werden gelöscht, Stick wird bootfähig")
		methvbox.pack_start(methpartradio, true, false, 1)
		methvbox.pack_start(methfullradio, true, false, 1)
		frames[1].add methvbox
		insthbox = Gtk::HBox.new(false, 3)
		instprog = Gtk::ProgressBar.new
		instprog.text = "Bitte klicken Sie auf \"Starten\""
		instbutton = Gtk::Button.new("Starten")
		insthbox.pack_start(instprog, true, true, 3)
		insthbox.pack_start(instbutton, false, false, 3)
		frames[2].add insthbox
		frames.each { |f|
			ovbox.pack_start(f, true, true, 3)
		}
		dialog.vbox.add(ovbox)
		dialog.signal_connect('response') { 
			check_memory
			dialog.destroy
		}
		instbutton.signal_connect('clicked') {
			[ instbutton, tgtsel, methfullradio, methpartradio, tgtreread ].each { |i| i.sensitive = false }
			dialog.set_response_sensitive(Gtk::Dialog::RESPONSE_OK, false)
			dialog.set_response_sensitive(Gtk::Dialog::RESPONSE_CLOSE, false)
			inst_success = run_installation(targets[tgtsel.active], methfullradio.active?, instprog)
			dialog.set_response_sensitive(Gtk::Dialog::RESPONSE_OK, true)
			dialog.set_response_sensitive(Gtk::Dialog::RESPONSE_CLOSE, true)
			[ instbutton, tgtsel, methfullradio, methpartradio, tgtreread ].each { |i| i.sensitive = true } if inst_success == false
		}
		dialog.show_all
	end
	
	def fill_drivelist(combo)
		drives = Array.new
		usable_drives = Array.new
		Dir.entries("/sys/block").each { |l|
			drives.push(MfsDiskDrive.new(l, true)) if l =~ /[a-z]$/ 
		}
		999.downto(0) { |n|
			begin
				combo.remove_text(n)
			rescue
			end
		}
		drives.each { |v|
			if v.size > 1_500_000_000 # && v.removable == true
				text =  v.vendor + " " + v.model + " " + v.human_size + " (" + v.device + ")"
				combo.append_text(text) 
				usable_drives.push(v)
			end
		}
		if usable_drives.size < 1
			combo.append_text "Kein geeignetes Laufwerk gefunden"
			combo.sensitive = false
		else
			combo.sensitive = true
		end
		combo.active = 0
		return usable_drives
	end
	
	def run_installation(tgt, boot, pgbar)
		puts "Target: " + tgt.vendor + " " + tgt.model + " " + tgt.human_size + " (" + tgt.device + ")"
		puts "Bootable: " + boot.to_s
		if boot == false
			copy_signatures(tgt)
		else
			run_partitioning(tgt)
			newtgt = MfsDiskDrive.new(tgt.device)
			write_boot(newtgt)
			write_sys(newtgt)
			copy_signatures(newtgt)
		end
	end
	
	def run_partitioning(tgt, pgbar=nil)
		sysmount = nil
		# Find out where the system is mounted
		if system("mountpoint -q /lesslinux/isoloop")
			sysmount = "/lesslinux/isoloop"
		elsif system("mountpoint -q /lesslinux/toram")
			sysmount = "/lesslinux/toram"
		elsif system("mountpoint -q /lesslinux/cdrom")
			sysmount = "/lesslinux/cdrom"
		end
		# FIXME! Usable error message!
		return false if sysmount.nil?
		# Get size of boot folder
		bootsize = -1
		szline = ` du -k #{sysmount} | tail -n1 ` 
		# FIXME! Usable error message!
		return false if szline.strip.size < 1
		bootsize = szline.strip.split[0].to_i * 1024 
		# Get size of ISO image
		isosize = -1
		isoline = ` df -k #{sysmount} | tail -n1 `
		isosize = isoline.strip.split[2].to_i * 1024 
		# Delete everything
		system("dd if=/dev/zero bs=1024 count=128 of=/dev/" + tgt.device)
		# Write an MBR
		system("parted -s /dev/#{tgt.device} unit B mklabel msdos")
		# Calculate partition sizes:
		p3blocks = ( isosize.to_f * 1.16 / 1048576 ).to_i + 1
		p2blocks = ( bootsize.to_f * 1.51 / 1048576 ).to_i + 1
		p1blocks = ( tgt.size / 1048576 ).to_i - p3blocks - p2blocks - 1 
		# Write the first partition (DOS)
		p2start = ( p1blocks + 1 ) * 1048576 
		system("parted -s /dev/#{tgt.device} unit B mkpart primary fat32 1048576 #{ (p2start - 1).to_s } ")
		system("parted -s /dev/#{tgt.device} unit B set 1 lba  on ")		
		# Write the second partition (boot)
		p3start = ( p1blocks + p2blocks + 1 ) * 1048576 
		p3end = ( p1blocks + p2blocks + p3blocks + 1 ) * 1048576 
		system("parted -s /dev/#{tgt.device} unit B mkpart primary ext2 #{p2start.to_s} #{ (p3start - 1).to_s } ")
		system("parted -s /dev/#{tgt.device} unit B set 2 boot  on ")
		# Write the third partition (ISO9660, system)
		system("parted -s /dev/#{tgt.device} unit B mkpart primary ext2 #{p3start.to_s} #{ (p3end - 1).to_s } ")
		# Force reread of partition tables
		system("partprobe /dev/#{tgt.device}")
		# Make filesystems
		system("mkfs.msdos -F32 /dev/#{tgt.device}1")
		system("mkfs.ext3 /dev/#{tgt.device}2")
	end
	
	def write_boot(tgt)
		sysmount = nil
		# Find out where the system is mounted
		if system("mountpoint -q /lesslinux/isoloop")
			sysmount = "/lesslinux/isoloop"
		elsif system("mountpoint -q /lesslinux/toram")
			sysmount = "/lesslinux/toram"
		elsif system("mountpoint -q /lesslinux/cdrom")
			sysmount = "/lesslinux/cdrom"
		end
		# FIXME! Usable error message!
		return false if sysmount.nil?
		# Write MBR
		system("dd if=/usr/share/syslinux/mbr.bin of=/dev/#{tgt.device}")
		# Copy content of boot partition
		tgt.partitions[1].mount("rw")
		system("rsync -avHP #{sysmount}/boot/ #{tgt.partitions[1].mount_point[0]}/boot/ ")
		# Write bootloader
		system("extlinux -i #{tgt.partitions[1].mount_point[0]}/boot/isolinux/ ")
		# Copy extlinux.conf if necessary
		unless File.exists?(tgt.partitions[1].mount_point[0] + "/boot/isolinux/extlinux.conf")
			system("cp #{tgt.partitions[1].mount_point[0]}/boot/isolinux/isolinux.cfg #{tgt.partitions[1].mount_point[0]}/boot/isolinux/extlinux.conf")
		end
		# Write UUID to config files
		system("sed -i 's/uuid=all/uuid=#{tgt.partitions[1].uuid}/g' #{tgt.partitions[1].mount_point[0]}/boot/isolinux/extlinux.conf")
		Dir.foreach("#{tgt.partitions[1].mount_point[0]}/boot/isolinux") { |n|
			if n =~ /cfg$/ 
				system("sed -i 's/uuid=all/uuid=#{tgt.partitions[1].uuid}/g' #{tgt.partitions[1].mount_point[0]}/boot/isolinux/#{n}")
			end
		}
		# Umount
		tgt.partitions[1].umount
	end
	
	def write_sys(tgt)
		sysmount = nil
		# Find out where the system is mounted
		if system("mountpoint -q /lesslinux/isoloop")
			sysmount = "/lesslinux/isoloop"
		elsif system("mountpoint -q /lesslinux/toram")
			sysmount = "/lesslinux/toram"
		elsif system("mountpoint -q /lesslinux/cdrom")
			sysmount = "/lesslinux/cdrom"
		end
		puts sysmount.to_s
		# FIXME! Usable error message!
		return false if sysmount.nil?
		mntline = ` df -k #{sysmount} | tail -n1 ` 
		puts mntline
		sysdev = mntline.strip.split[0]
		puts sysdev
		# FIXME: use seek and skip in dd loop to get usable progress
		ddstr = "dd if=#{sysdev} of=/dev/#{tgt.device}3"
		puts ddstr
		system(ddstr)
	end
	
	# FIXME! Show errors also in GUI
	# FIXME! Remove hardcoded size!
	# FIXME! Source might be /lesslinux/cdrom/Antibot3 instead of /lesslinux/cdrom
	def copy_signatures(tgt)
		# Are there partitions?
		if tgt.partitions.size < 1
			puts "ERROR: No partitions found!"
			return false
		end
		# Is the partition mountable - free space should answer that?
		free = tgt.partitions[0].free_space 
		if free.nil? ||  free < 120_000_000
			puts "ERROR: Not enough free space!"
			return false
		end
		# Can we unmount?
		unless tgt.partitions[0].mount_point.nil?
			unless tgt.partitions[0].umount
				puts "ERROR: Could not umount!"
				return false
			end
		end
		source = nil
		if File.exist?("/lesslinux/cdrom/antibot3/avupdate")
			source = "/lesslinux/cdrom/antibot3"
		elsif File.exist?("/lesslinux/cdrom/avupdate")
			source = "/lesslinux/cdrom"
		elsif File.exist?("/lesslinux/isoloop/antibot3/avupdate")
			source = "/lesslinux/isoloop/antibot3"
		elsif File.exist?("/lesslinux/isoloop/avupdate")
			source = "/lesslinux/isoloop"
		else
			puts "ERROR: Source not found!"
			return false
		end
		# Now mount and install
		tgt.partitions[0].mount("rw")
		system("mkdir -p '" + tgt.partitions[0].mount_point[0] + "/antibot3/antivir'")
		system("tar -C " + source + " -cvf - avupdate | tar -C '" + tgt.partitions[0].mount_point[0] + "/antibot3' -xf - ")
		# FIXME: Progress bar here, move to VTE!
		system("tar -C /AntiVir/ -cvf - . | tar -C '" + tgt.partitions[0].mount_point[0] + "/antibot3/antivir/' -xf - ")
		tgt.partitions[0].umount
		system("umount /AntiVir")
		system("mkdir -p /lesslinux/antibot")
		tgt.partitions[0].mount("rw", "/lesslinux/antibot")
		system("mount --bind /lesslinux/antibot/antibot3/antivir /AntiVir")
	end
	
	
end